`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 04/28/2020 09:32:08 PM
// Design Name: 
// Module Name: speaker
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module Lab10_2(
    f_crystal, // clock from crystal
    rst, // active low reset
    PS2_DATA,
    PS2_CLK,
    audio_mclk, // master clock
    audio_lrck, // left-right clock
    audio_sck, // serial clock
    audio_sdin, // serial audio data input
    Caps, 
    D_ssd,
    ssd_ctl,
    uppercase
    );
    // I/O declaration
    input f_crystal; // clock from the crystal
    input rst; // active low reset
    output reg uppercase;
    output audio_mclk; // master clock
    output audio_lrck; // left-right clock
    output audio_sck; // serial clock
    output audio_sdin; // serial audio data input
    output Caps;
    output [7:0]D_ssd;
    output [3:0]ssd_ctl;
    inout PS2_DATA, PS2_CLK;
    wire clk_1;
    wire [21:0]note_div;
    // Declare internal nodes
    wire [15:0] audio_left, audio_right;
    //Keyboard related
    wire [511:0]key_down;
    wire [8:0]last_change;
    wire key_valid;
    wire [3:0]converter_out;
    reg change_caps, shift;
    wire [3:0]Display;
    frq_div_clk U0(
    .f_out(clk_1), // divided clock output
    .f_crystal(f_crystal), // global clock input
    .rst(rst) // active low reset
    );
    KeyboardDecoder U1(
    .key_down(key_down),
    .last_change(last_change),
    .key_valid(key_valid),
    .PS2_DATA(PS2_DATA),
    .PS2_CLK(PS2_CLK),
    .rst(rst),
    .clk(f_crystal)
    );
    always @* //(posedge f_crystal or posedge rst)
        if(rst)
            change_caps = 1'b0;
        else if(key_down[88] && key_valid) //Caps is pressed   And use key_valid to detect the Impulse
            change_caps = 1'b1;
        else
            change_caps = 1'd0;
    always @* //(posedge f_crystal or posedge rst)
        if(rst)
            shift = 1'b0;
        else if((key_down[18] || key_down[89])) //Two shift key
            shift = 1'b1;
        else
            shift = 1'd0;
    FSM U2(
    .in(change_caps), //input control
    .clk(f_crystal), // global clock signal
    .rst(rst), // high active reset
    .state(Caps)
    );
    always @*
        uppercase = ((Caps == 1'b0) & (shift == 1'b1)) | ((Caps == 1'b1) & (shift == 1'b0)); 
    Keyboard_converter U3(
    .in(last_change),
    .out(converter_out),
    .uppercase(uppercase),
    .key_down(key_down)
    );
    note_MUX U4(
    .in(converter_out),
    .note_div(note_div),
    .display(Display)
    );
    assign ssd_ctl = 4'b1110;
    Display_7_segment U5(.D_ssd(D_ssd), .i(Display));
    // Note generation
    note_gen Ung(
    .clk(f_crystal), // clock from crystal
    .rst(rst), // active low reset
    .note_div(note_div), // div for note generation
    .audio_left(audio_left), // left sound audio
    .audio_right(audio_right) // right sound audio
    );
    // Speaker controllor
    speaker_control Usc(
    .f_crystal(f_crystal), // clock from the crystal
    .rst(rst), // active low reset
    .audio_left(audio_left), // left channel audio data input
    .audio_right(audio_right), // right channel audio data input
    .audio_mclk(audio_mclk), // master clock
    .audio_lrck(audio_lrck), // left-right clock
    .audio_sck(audio_sck), // serial clock
    .audio_sdin(audio_sdin) // serial audio data input
    );
endmodule
